/*************************************************************************************
 * @copyright: Huawei Technologies Co. Ltd.                                          *
 * @file: Greedy.h                                                                   *
 * @author: Xu Le, He Jianfan                                                        *
 * This file is part of 2017 Huawei CodeCraft solution <http://codecraft.huawei.com> *
 *************************************************************************************/
#ifndef __GREEDY_H__
#define __GREEDY_H__

#include "TopologyScanner.h"

/** Greedy algorithm implemented by a finite state machine(FSM). */
class Greedy
{
public:
	Greedy(Graph& _graph, std::list<std::vector<int> >& _serverGroups);
	~Greedy();

	Greedy(const Greedy& rhs) = delete;
	Greedy& operator=(const Greedy& rhs) = delete;

	void initialize(std::list<int>& servers);
	bool optimize(std::list<int>& servers, long justResultFee);

	void obtainOutLinkCap(int *_out_link_cap);
	void obtainRentalDeployFeeRatio(double *_rental_deploy_fee_ratio);

private:
	enum Progress {
		ZOM, ///< Zero Order Minus
		ZOP, ///< Zero Order Plus
		FOP, ///< First Order Plus
		FOM, ///< First Order Minus
		EX   ///< Exchange Operation
	};

	void _sortCspNode(double *_rental_deploy_fee_ratio);
	uint32_t _selectRestNode(bool onlyAbandon);

private:
	Graph &graph;

	std::set<int> definiteServers;     ///< set of access nodes that have to deploy a cdn server.
	std::unordered_set<int> accessSet; ///< set of all access nodes.

	Progress progress;   ///< indicates current optimize progress.

	int curIdx;          ///< current possible consumption node index.
	int curNBIdx;        ///< current possible selected neighbor node index.
	uint32_t cspCN;      ///< only count those possible consumption node.
	uint32_t totalCN;    ///< only count those possible consumption node before ZOM progress.
	uint32_t neighborCN; ///< only count those possible neighbor node selected by _selectRestNode().

	bool *deployed;      ///< if node x has deployed a server, then deployed[i] is true, otherwise false.
	int *cspRank;        ///< consumption rank used in zero order plus and zero order minus progress.
	int *neighborRank;   ///< neighbor rank used in first order plus progress.
	int *outLinkCap;     ///< the sum up capacity of all links of a network node, it is a point to TopologyScanner::outLinkCap.
	double *cspRDFR;     ///< when an access node doesn't deploy server, it will need a minimum link rental fee X, this ratio value certainly is X/singleServerFee.

	bool hasExecutedZOP;  ///< whether ZOP progress has been executed.
	bool rightAfterExchange; ///< whether we have just executed an exchange operation.
	bool inExchangeProgress; ///< whether current zero order minus operation is a sub operation of exchange operation.
	int lastChangedNode;  ///< store the node in previous minus operation for undo operation.
	int tryExchangeTimes; ///< store the number we have tried to exchange, it is used to resume curIdx when return to exchange operation.
	long lastResultFee;   ///< store the total fee result by previous operation for comparsion purpose.

	std::list<int>::iterator itS; ///< iterate servers list for remove a certain server.
};

#endif /* __GREEDY_H__ */
