/*************************************************************************************
 * @copyright: Huawei Technologies Co. Ltd.                                          *
 * @file: Log.cpp                                                                    *
 * @author: Xu Le                                                                    *
 * This file is part of 2017 Huawei CodeCraft solution <http://codecraft.huawei.com> *
 *************************************************************************************/
#include "Log.h"

int log_level = 3;

#ifdef __GNUC__
const char* getLogLevel()
{
	switch (log_level)
	{
	case 1:
		return "ERROR";
	case 2:
		return "WARNNING";
	case 3:
		return "INFO";
	case 4:
		return "DEBUG";
	default:
		return "UNKNOWN";
	}
}
#else
static void printudec(unsigned int udec)
{
	if (udec == 0)
		return;
	printudec(udec/10);
	putchar((char)(udec%10 + '0'));
}

static void printdec(int dec)
{
	if (dec < 0)
	{
		putchar('-');
		dec = -dec;
	}
	printudec((unsigned int)dec);
}

static void printdbl(double dbl)
{
	int tmpint = (int)dbl;
	printdec(tmpint);
	putchar('.');
	dbl -= tmpint;
	tmpint = (int)(dbl * 1000000);
	printdec(tmpint);
}

static void printstr(const char *str)
{
	while (*str)
		putchar(*str++);
}

static void printbin(int bin)
{
	if (bin == 0)
	{
		printstr("0b");
		return;
	}
	printbin(bin/2);
	putchar((char)(bin%2 + '0'));
}

static void printhex(int hex)
{
	if (hex == 0)
	{
		printstr("0x");
		return;
	}
	printhex(hex/16);
	if (hex < 10)
		putchar((char)(hex%16 + '0'));
	else
		putchar((char)(hex%16 - 10 + 'a' ));
}

#if LOG_NEED_MACRO_FILE_LINE
void log(LogLevel level, const char *file, const int line, const char *fmt, ...)
#else
void log(LogLevel level, const char *fmt, ...)
#endif
{
	switch (level)
	{
	case LogLevel::uncond:
#if LOG_NEED_MACRO_FILE_LINE
		printf("%s: %d | ", file, line);
#endif
		break;
	case LogLevel::error:
		if (log_level >= static_cast<int>(LogLevel::error))
#if LOG_NEED_MACRO_FILE_LINE
			printf("ERROR | %s: %d | ", file, line);
#else
			printf("ERROR | ");
#endif
		else
			return;
		break;
	case LogLevel::warning:
		if (log_level >= static_cast<int>(LogLevel::warning))
#if LOG_NEED_MACRO_FILE_LINE
			printf("WARN | %s: %d | ", file, line);
#else
			printf("WARN | ");
#endif
		else
			return;
		break;
	case LogLevel::info:
		if (log_level >= static_cast<int>(LogLevel::info))
#if LOG_NEED_MACRO_FILE_LINE
			printf("INFO | %s: %d | ", file, line);
#else
			printf("INFO | ");
#endif
		else
			return;
		break;
	case LogLevel::debug:
		if (log_level >= static_cast<int>(LogLevel::debug))
#if LOG_NEED_MACRO_FILE_LINE
			printf("DEBUG | %s: %d | ", file, line);
#else
			printf("DEBUG | ");
#endif
		else
			return;
		break;
	default:
		fprintf(stderr, "Log level undefined!\n");
		return;
	}

	va_list vl;
	va_start(vl, fmt);

	char vargch = 0;
	int vargint = 0;
	unsigned int varguint = 0;
	double vargdbl = 0.0;
	char *vargpch = NULL;

	char *pfmt = const_cast<char*>(fmt);

	while (*pfmt)
	{
		if (*pfmt == '%')
		{
			switch (*(++pfmt))
			{
			case 'c':
				vargch = va_arg(vl, int); 
				putchar(vargch);
				break;
			case 'd':
			case 'i':
				vargint = va_arg(vl, int);
				printdec(vargint);
				break;
			case 'u':
				varguint = va_arg(vl, unsigned int);
				printudec(varguint);
				break;
			case 'f':
				vargdbl = va_arg(vl, double);
				printdbl(vargdbl);
				break;
			case 's':
				vargpch = va_arg(vl, char*);
				printstr(vargpch);
				break;
			case 'b':
			case 'B':
				vargint = va_arg(vl, int);
				printbin(vargint);
				break;
			case 'x':
			case 'X':
				vargint = va_arg(vl, int);
				printhex(vargint);
				break;
			case '%':
				putchar('%');
				break;
			default:
				break;
			}
			++pfmt;
		}
		else
		{
			putchar(*pfmt++);
		}
	}

	va_end(vl);
}
#endif
