/*************************************************************************************
 * @copyright: Huawei Technologies Co. Ltd.                                          *
 * @file: Log.h                                                                      *
 * @author: Xu Le                                                                    *
 * This file is part of 2017 Huawei CodeCraft solution <http://codecraft.huawei.com> *
 *************************************************************************************/
#ifndef __LOG_H__
#define __LOG_H__

#define _CRT_SECURE_NO_WARNINGS
#include <stdarg.h>
#include <stdio.h>

#define LOG_NEED_MACRO_FILE_LINE    0

#ifdef __GNUC__
#define ERROR_LEVEL      1
#define WARNING_LEVEL    2
#define INFO_LEVEL       3
#define DEBUG_LEVEL      4

#if LOG_NEED_MACRO_FILE_LINE
#define uncond_log(...)     do { printf("%s: %d | ", __FILE__, __LINE__); printf(__VA_ARGS__); } while(0)
#define error_log(...)      do { if (log_level >= ERROR_LEVEL)   { printf("ERROR | %s: %d | ", __FILE__, __LINE__); printf(__VA_ARGS__); } } while(0)
#define warning_log(...)    do { if (log_level >= WARNING_LEVEL) { printf("WARN | %s: %d | ", __FILE__, __LINE__); printf(__VA_ARGS__); } } while(0)
#define info_log(...)       do { if (log_level >= INFO_LEVEL)    { printf("INFO | %s: %d | ", __FILE__, __LINE__); printf(__VA_ARGS__); } } while(0)
#define debug_log(...)      do { if (log_level >= DEBUG_LEVEL)   { printf("DEBUG | %s: %d | ", __FILE__, __LINE__); printf(__VA_ARGS__); } } while(0)
#else
#define uncond_log(...)     do { printf(__VA_ARGS__); } while(0)
#define error_log(...)      do { if (log_level >= ERROR_LEVEL)   { printf("ERROR | "); printf(__VA_ARGS__); } } while(0)
#define warning_log(...)    do { if (log_level >= WARNING_LEVEL) { printf("WARN | "); printf(__VA_ARGS__); } } while(0)
#define info_log(...)       do { if (log_level >= INFO_LEVEL)    { printf("INFO | "); printf(__VA_ARGS__); } } while(0)
#define debug_log(...)      do { if (log_level >= DEBUG_LEVEL)   { printf("DEBUG | "); printf(__VA_ARGS__); } } while(0)
#endif
const char* getLogLevel();
#else /* __GNUC__ */
enum class LogLevel {
	uncond,
	error,
	warning,
	info,
	debug
};

#if LOG_NEED_MACRO_FILE_LINE
void log(LogLevel level, const char *file, const int line, const char *fmt, ...);
#else
void log(LogLevel level, const char *fmt, ...);
#endif
#endif

#endif /* __LOG_H__ */
