/*************************************************************************************
 * @copyright: Huawei Technologies Co. Ltd.                                          *
 * @file: Timer.h                                                                    *
 * @author: Xu Le                                                                    *
 * This file is part of 2017 Huawei CodeCraft solution <http://codecraft.huawei.com> *
 *************************************************************************************/
#ifndef __TIMER_H__
#define __TIMER_H__

#define RUNNING_TIME_HIGH_PRECISION    1

#if defined(__GNUC__) && RUNNING_TIME_HIGH_PRECISION
#include <sys/time.h>
#else
#include <time.h>
#endif

#include <string>
#include "Log.h"

#if defined(__GNUC__) && RUNNING_TIME_HIGH_PRECISION
class Timer
{
public:
	explicit Timer(const char *s) : _note(s) { gettimeofday(&_start_time, NULL); }
	~Timer()
	{
		struct timeval _end_time;
		gettimeofday(&_end_time, NULL);
		printf("%ssecTime = %lds, usecTime = %ldus.\n", _note.c_str(), _end_time.tv_sec - _start_time.tv_sec, _end_time.tv_usec - _start_time.tv_usec);
	}
	
	double elapsed()
	{
		struct timeval _cur_time;
		gettimeofday(&_cur_time, NULL);
		double secTime = _cur_time.tv_sec - _start_time.tv_sec;
		double usecTime = _cur_time.tv_usec - _start_time.tv_usec;
		return secTime + usecTime/1000000;
	}

private:
	Timer(const Timer&);
	Timer& operator=(const Timer&);

private:
	std::string _note;
	struct timeval _start_time;
};
#else
class Timer
{
public:
	explicit Timer(const char *s) : _note(s) { _start_time = clock(); }
	~Timer() { printf("%s%fs.\n", _note.c_str(), elapsed()); }
	
	inline double elapsed() { return static_cast<double>(clock() - _start_time) / CLOCKS_PER_SEC; }

private:
	Timer(const Timer&);
	Timer& operator=(const Timer&);

private:
	std::string _note;
	clock_t _start_time;
};
#endif

#endif /* __TIMER_H__ */
