/*************************************************************************************
 * @copyright: Huawei Technologies Co. Ltd.                                          *
 * @file: Utils.cpp                                                                  *
 * @author: Xu Le                                                                    *
 * This file is part of 2017 Huawei CodeCraft solution <http://codecraft.huawei.com> *
 *************************************************************************************/
#include "Utils.h"

extern int log_level;

extern int maxUnitRentalFee;

extern uint32_t networkNodeNum;
extern uint32_t linkNum;
extern uint32_t consumptionNodeNum;
extern uint32_t serverDeploymentFee;

extern std::vector<int> accessTable;
extern std::vector<int> demandTable;

void parseInput(char *filename, std::list<struct LinkTuple>& linkTuples)
{
	FILE *fp = fopen(filename, "r");
	if (fp == NULL)
	{
		error_log("Fail to open file %s.\n", filename);
		exit(EXIT_FAILURE);
	}

	int srcNode = 0, dstNode = 0, bandwidth = 0, rental = 0;
	int cspID = 0, netID = 0, demand = 0;
	fscanf(fp, "%u %u %u", &networkNodeNum, &linkNum, &consumptionNodeNum);
	fgetc(fp);
	fscanf(fp, "%u", &serverDeploymentFee);
	fgetc(fp);
	accessTable.resize(consumptionNodeNum);
	demandTable.resize(consumptionNodeNum);

	uint32_t _linkNum = linkNum;
	uint32_t _consumptionNodeNum = consumptionNodeNum;
	while (_linkNum-- > 0)
	{
		fscanf(fp, "%d %d %d %d", &srcNode, &dstNode, &bandwidth, &rental);
		linkTuples.emplace_back(srcNode, dstNode, bandwidth, rental);
		if (maxUnitRentalFee < rental)
			maxUnitRentalFee = rental;
	}
	fgetc(fp);
	while (_consumptionNodeNum-- > 0)
	{
		fscanf(fp, "%d %d %d", &cspID, &netID, &demand);
		accessTable[cspID] = netID;
		demandTable[cspID] = demand;
	}

	fclose(fp);
}

void outputToFile(char *filename, std::list<std::vector<int> >& fluxPathList)
{
	FILE *fp = fopen(filename, "w");
	if (fp == NULL)
	{
		error_log("Fail to open file %s.\n", filename);
		exit(EXIT_FAILURE);
	}

	fprintf(fp, "%u\n\n", static_cast<uint32_t>(fluxPathList.size()));

	for (std::list<std::vector<int> >::iterator iter = fluxPathList.begin(); iter != fluxPathList.end(); ++iter)
	{
		std::vector<int> &fluxPath = *iter; // alias
		for (size_t i = 0; i < fluxPath.size()-1; ++i)
			fprintf(fp, "%d ", fluxPath[i]);
		fprintf(fp, "%d\n", fluxPath.back());
	}

	fclose(fp);
}

bool binarySearch(const std::vector<int>& vec, const int key)
{
    int low = 0, high = (int)vec.size()-1, mid;
    while (low <= high)
    {
        mid = low + ((high - low) >> 1);
        if (key == vec[mid])
            return true;
        else if (key < vec[mid])
            high = mid - 1;
        else
            low = mid + 1;
    }
    return false;
}

Complexity defineDefaultComplexity()
{
	if (consumptionNodeNum > 300)
		return Complexity::Hell;
	if (consumptionNodeNum > 200 || networkNodeNum > 800)
		return Complexity::UltraHard;
	if (consumptionNodeNum > 100 || networkNodeNum > 500)
		return Complexity::VeryHard;
	if (consumptionNodeNum > 50 || networkNodeNum > 300)
		return Complexity::Hard;
	if (consumptionNodeNum > 20 || networkNodeNum > 100)
		return Complexity::General;
	if (consumptionNodeNum > 10 || networkNodeNum > 50)
		return Complexity::Simple;
	return Complexity::Oversimplified;
}

const char* complexityString(Complexity complexity)
{
	switch (complexity)
	{
	case Complexity::Oversimplified:
		return "oversimplified";
	case Complexity::Simple:
		return "simple";
	case Complexity::General:
		return "general";
	case Complexity::Hard:
		return "hard";
	case Complexity::VeryHard:
		return "very-hard";
	case Complexity::UltraHard:
		return "ultra-hard";
	default:
		return "hell";
	}
}

////////////////    convenient functions of std::vector and std::list    ////////////////
void printVector(std::vector<int>& _vector, const char *_note)
{
	if (log_level >= INFO_LEVEL)
	{
		printf("%s", _note);
		for (std::vector<int>::iterator iter = _vector.begin(); iter != _vector.end(); ++iter)
			printf("%d ", *iter);
		printf("\n");
	}
}

void printVector(std::vector<double>& _vector, const char *_note)
{
	if (log_level >= INFO_LEVEL)
	{
		printf("%s", _note);
		for (std::vector<double>::iterator iter = _vector.begin(); iter != _vector.end(); ++iter)
			printf("%f ", *iter);
		printf("\n");
	}
}

void printList(std::list<int>& _list, const char *_note)
{
	if (log_level >= INFO_LEVEL)
	{
		printf("%s", _note);
		for (std::list<int>::iterator iter = _list.begin(); iter != _list.end(); ++iter)
			printf("%d ", *iter);
		printf("\n");
	}
}

void printList(std::list<double>& _list, const char *_note)
{
	if (log_level >= INFO_LEVEL)
	{
		printf("%s", _note);
		for (std::list<double>::iterator iter = _list.begin(); iter != _list.end(); ++iter)
			printf("%f ", *iter);
		printf("\n");
	}
}
