/*************************************************************************************
 * @copyright: Huawei Technologies Co. Ltd.                                          *
 * @file: Utils.h                                                                    *
 * @author: Xu Le                                                                    *
 * This file is part of 2017 Huawei CodeCraft solution <http://codecraft.huawei.com> *
 *************************************************************************************/
#ifndef __UTILS_H__
#define __UTILS_H__

#include <cstdint>
#include <cstring>
#include <vector>
#include <list>
#include <stack>
#include <queue>
#include <set>
#include <unordered_set>
#include <unordered_map>
#include <algorithm>
#include <utility>

#include "Timer.h"

struct LinkTuple
{
	LinkTuple(int _src, int _dst, int _bandwidth, int _rental) : src(_src), dst(_dst), bandwidth(_bandwidth), rental(_rental) {}

	LinkTuple(const LinkTuple& rhs) = default;
	LinkTuple& operator=(const LinkTuple& rhs) = default;

	int src;
	int dst;
	int bandwidth;
	int rental;
};

enum class Complexity {
	Oversimplified,
	Simple,
	General,
	Hard,
	VeryHard,
	UltraHard,
	Hell
};

void parseInput(char *filename, std::list<struct LinkTuple>& linkTuples);

void outputToFile(char *filename, std::list<std::vector<int> >& fluxPathList);

bool binarySearch(const std::vector<int>& vec, const int key);

Complexity defineDefaultComplexity();

const char* complexityString(Complexity complexity);

////////////////    convenient functions of std::vector and std::list    ////////////////
void printVector(std::vector<int>& _vector, const char *_note);

void printVector(std::vector<double>& _vector, const char *_note);

void printList(std::list<int>& _list, const char *_note);

void printList(std::list<double>& _list, const char *_note);

#endif /* __UTILS_H__ */
